from __future__ import absolute_import, print_function, division

import unittest

from pony.orm.core import *
from pony.orm.core import Entity
from pony.orm.tests.testutils import *
from pony.orm.tests import db_params


class TestDiag(unittest.TestCase):
    @raises_exception(ERDiagramError, 'Entity Entity1 already exists')
    def test_entity_duplicate(self):
        db = Database()
        class Entity1(db.Entity):
            id = PrimaryKey(int)
        class Entity1(db.Entity):
            id = PrimaryKey(int)

    @raises_exception(ERDiagramError, 'Interrelated entities must belong to same database.'
                                    ' Entities Entity2 and Entity1 belongs to different databases')
    def test_diagram1(self):
        db = Database()
        class Entity1(db.Entity):
            id = PrimaryKey(int)
            attr1 = Required('Entity2')
        db.bind(**db_params)
        db = Database()
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Optional(Entity1)
        db.bind(**db_params)
        db.generate_mapping()

    @raises_exception(ERDiagramError, 'Entity definition Entity2 was not found')
    def test_diagram2(self):
        db = Database()
        class Entity1(db.Entity):
            id = PrimaryKey(int)
            attr1 = Required('Entity2')
        db.bind(**db_params)
        db.generate_mapping()

    @raises_exception(TypeError, 'Entity1._table_ property must be a string. Got: 123')
    def test_diagram3(self):
        db = Database()
        class Entity1(db.Entity):
            _table_ = 123
            id = PrimaryKey(int)
        db.bind(**db_params)
        db.generate_mapping()

    def test_diagram4(self):
        db = Database()
        class Entity1(db.Entity):
            id = PrimaryKey(int)
            attr1 = Set('Entity2', table='Table1')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Set(Entity1, table='Table1')
        db.bind(**db_params)
        db.generate_mapping(create_tables=True)
        db.drop_all_tables(with_all_data=True)

    def test_diagram5(self):
        db = Database()
        class Entity1(db.Entity):
            id = PrimaryKey(int)
            attr1 = Set('Entity2')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Required(Entity1)
        db.bind(**db_params)
        db.generate_mapping(create_tables=True)
        db.drop_all_tables(with_all_data=True)

    @raises_exception(MappingError, "Parameter 'table' for Entity1.attr1 and Entity2.attr2 do not match")
    def test_diagram6(self):
        db = Database()
        class Entity1(db.Entity):
            id = PrimaryKey(int)
            attr1 = Set('Entity2', table='Table1')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Set(Entity1, table='Table2')
        db.bind(**db_params)
        db.generate_mapping()

    @raises_exception(MappingError, 'Table name "Table1" is already in use')
    def test_diagram7(self):
        db = Database()
        class Entity1(db.Entity):
            _table_ = 'Table1'
            id = PrimaryKey(int)
            attr1 = Set('Entity2', table='Table1')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Set(Entity1, table='Table1')
        db.bind(**db_params)
        db.generate_mapping()

    def test_diagram8(self):
        db = Database()
        class Entity1(db.Entity):
            id = PrimaryKey(int)
            attr1 = Set('Entity2')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Set(Entity1)
        db.bind(**db_params)
        db.generate_mapping(create_tables=True)
        if pony.__version__ >= '0.9':
            m2m_table = db.schema.tables['entity1_attr1']
            col_names = set(m2m_table.columns)
            self.assertEqual(col_names, {'entity1_id', 'entity2_id'})
            m2m_columns = [c.name for c in Entity1.attr1.meta.m2m_columns]
            self.assertEqual(m2m_columns, ['entity1_id'])
        else:
            table_name = 'Entity1_Entity2' if db.provider.dialect == 'SQLite' else 'entity1_entity2'
            m2m_table = db.schema.tables[table_name]
            col_names = {col.name for col in m2m_table.column_list}
            self.assertEqual(col_names, {'entity1', 'entity2'})
            self.assertEqual(Entity1.attr1.get_m2m_columns(), ['entity1'])
        db.drop_all_tables(with_all_data=True)

    def test_diagram9(self):
        db = Database()
        class Entity1(db.Entity):
            a = Required(int)
            b = Required(str)
            PrimaryKey(a, b)
            attr1 = Set('Entity2')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Set(Entity1)
        db.bind(**db_params)
        db.generate_mapping(create_tables=True)
        if pony.__version__ >= '0.9':
            m2m_table = db.schema.tables['entity1_attr1']
            col_names = {col for col in m2m_table.columns}
            self.assertEqual(col_names, {'entity1_a', 'entity1_b', 'entity2_id'})
        else:
            table_name = 'Entity1_Entity2' if db.provider.dialect == 'SQLite' else 'entity1_entity2'
            m2m_table = db.schema.tables[table_name]
            col_names = set([col.name for col in m2m_table.column_list])
            self.assertEqual(col_names, {'entity1_a', 'entity1_b', 'entity2'})
        db.drop_all_tables(with_all_data=True)

    def test_diagram10(self):
        db = Database()
        class Entity1(db.Entity):
            a = Required(int)
            b = Required(str)
            PrimaryKey(a, b)
            attr1 = Set('Entity2', column='z')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Set(Entity1, columns=['x', 'y'])
        db.bind(**db_params)
        db.generate_mapping(create_tables=True)
        db.drop_all_tables(with_all_data=True)

    @raises_exception(MappingError, 'Invalid number of columns for Entity2.attr2')
    def test_diagram11(self):
        db = Database()
        class Entity1(db.Entity):
            a = Required(int)
            b = Required(str)
            PrimaryKey(a, b)
            attr1 = Set('Entity2', column='z')
        class Entity2(db.Entity):
            id = PrimaryKey(int)
            attr2 = Set(Entity1, columns=['x'])
        db.bind(**db_params)
        db.generate_mapping()

    @raises_exception(ERDiagramError, 'Base Entity does not belong to any database')
    def test_diagram12(self):
        class Test(Entity):
            name = Required(unicode)

    @raises_exception(ERDiagramError, 'Entity class name should start with a capital letter. Got: entity1')
    def test_diagram13(self):
        db = Database()
        class entity1(db.Entity):
            a = Required(int)
        db.bind(**db_params)
        db.generate_mapping()

if __name__ == '__main__':
    unittest.main()


from __future__ import absolute_import, print_function, division

import unittest

from pony.orm.core import *
from pony.orm.tests.testutils import raises_exception
from pony.orm.tests import db_params, only_for

@only_for(providers=['cockroach'])
class TestDiag(unittest.TestCase):
    @raises_exception(TypeError, '`interleave` option cannot be specified for Set attribute Foo.x')
    def test_1(self):
        db = Database()
        class Foo(db.Entity):
            x = Set('Bar', interleave=True)
        class Bar(db.Entity):
            y = Required('Foo')

    @raises_exception(TypeError, "`interleave` option value should be True, False or None. Got: 'yes'")
    def test_2(self):
        db = Database()
        class Foo(db.Entity):
            x = Required('Bar', interleave='yes')
        class Bar(db.Entity):
            y = Set('Foo')

    @raises_exception(TypeError, 'only one attribute may be marked as interleave. Got: Foo.x, Foo.y')
    def test_3(self):
        db = Database()
        class Foo(db.Entity):
            x = Required(int, interleave=True)
            y = Required(int, interleave=True)

    @raises_exception(TypeError, 'Interleave attribute should be part of relationship. Got: Foo.x')
    def test_4(self):
        db = Database()
        class Foo(db.Entity):
            x = Required(int, interleave=True)

    def test_5(self):
        db = Database(**db_params)
        class Bar(db.Entity):
            y = Set('Foo')

        class Foo(db.Entity):
            x = Required('Bar', interleave=True)
            id = Required(int)
            PrimaryKey(x, id)

        db.generate_mapping(create_tables=True)
        s = ') INTERLEAVE IN PARENT "bar" ("x")'
        self.assertIn(s, db.schema.tables['foo'].get_create_command())
        db.drop_all_tables()


if __name__ == '__main__':
    unittest.main()


import unittest

from pony import orm
from pony.orm.tests import setup_database, teardown_database

db = orm.Database()


class Person(db.Entity):
    name = orm.Required(str)


class Test(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        setup_database(db)

    @classmethod
    def tearDownClass(cls):
        teardown_database(db)

    def test_1(self):
        with orm.db_session:
            a = Person(name='John')
            a.delete()
            Person.exists(name='Mike')



from .app import app
from .models import db
from flask import render_template, request, flash, redirect, abort
from flask_login import current_user, logout_user, login_user, login_required
from datetime import datetime
from pony.orm import flush

@app.route('/')
def index():
    users = db.User.select()
    return render_template('index.html', user=current_user, users=users)

@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        possible_user = db.User.get(login=username)
        if not possible_user:
            flash('Wrong username')
            return redirect('/login')
        if possible_user.password == password:
            possible_user.last_login = datetime.now()
            login_user(possible_user)
            return redirect('/')

        flash('Wrong password')
        return redirect('/login')
    else:
        return render_template('login.html')
    
@app.route('/reg', methods=['GET', 'POST'])
def reg():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        exist = db.User.get(login=username)
        if exist:
            flash('Username %s is already taken, choose another one' % username)
            return redirect('/reg')

        user = db.User(login=username, password=password)
        user.last_login = datetime.now()
        flush()
        login_user(user)
        flash('Successfully registered')
        return redirect('/')
    else:
        return render_template('reg.html')

@app.route('/logout')
@login_required
def logout():
    logout_user()
    flash('Logged out')
    return redirect('/')


import unittest

from pony.thirdparty.compiler.transformer import parse
from pony.orm.decompiling import Decompiler
from pony.orm.asttranslation import ast2src


def generate_gens():
    patterns = [
        '(x * y) * [z * j)',
        '([x * y) * z) * j',
        '(x * [y * z)) * j',
        'x * ([y * z) * j)',
        'x * (y * [z * j))'
    ]

    ops = ('and', 'or')
    nots = (True, False)

    result = []

    for pat in patterns:
        cur = pat
        for op1 in ops:
            for op2 in ops:
                for op3 in ops:
                    res = cur.replace('*', op1, 1)
                    res = res.replace('*', op2, 1)
                    res = res.replace('*', op3, 1)
                    result.append(res)

    final = []

    for res in result:
        for par1 in nots:
            for par2 in nots:
                for a in nots:
                    for b in nots:
                        for c in nots:
                            for d in nots:
                                cur = res.replace('(', 'not(') if not par1 else res
                                if not par2:
                                    cur = cur.replace('[', 'not(')
                                else:
                                    cur = cur.replace('[', '(')
                                if not a: cur = cur.replace('x', 'not x')
                                if not b: cur = cur.replace('y', 'not y')
                                if not c: cur = cur.replace('z', 'not z')
                                if not d: cur = cur.replace('j', 'not j')
                                final.append(cur)

    return final

def create_test(gen):
    def wrapped_test(self):
        def get_condition_values(cond):
            result = []
            vals = (True, False)
            for x in vals:
                for y in vals:
                    for z in vals:
                        for j in vals:
                            result.append(eval(cond, {'x': x, 'y': y, 'z': z, 'j': j}))
            return result
        src1 = '(a for a in [] if %s)' % gen
        src2 = 'lambda x, y, z, j: (%s)' % gen
        src3 = '(m for m in [] if %s for n in [] if %s)' % (gen, gen)

        code1 = compile(src1, '<?>', 'eval').co_consts[0]
        ast1 = Decompiler(code1).ast
        res1 = ast2src(ast1).replace('.0', '[]')
        res1 = res1[res1.find('if')+2:-1]

        code2 = compile(src2, '<?>', 'eval').co_consts[0]
        ast2 = Decompiler(code2).ast
        res2 = ast2src(ast2).replace('.0', '[]')
        res2 = res2[res2.find(':')+1:]

        code3 = compile(src3, '<?>', 'eval').co_consts[0]
        ast3 = Decompiler(code3).ast
        res3 = ast2src(ast3).replace('.0', '[]')
        res3 = res3[res3.find('if')+2: res3.rfind('for')-1]

        if get_condition_values(gen) != get_condition_values(res1):
            self.fail("Incorrect generator decompilation: %s -> %s" % (gen, res1))

        if get_condition_values(gen) != get_condition_values(res2):
            self.fail("Incorrect lambda decompilation: %s -> %s" % (gen, res2))

        if get_condition_values(gen) != get_condition_values(res3):
            self.fail("Incorrect multi-for generator decompilation: %s -> %s" % (gen, res3))

    return wrapped_test


class TestDecompiler(unittest.TestCase):
    pass


for i, gen in enumerate(generate_gens()):
    test_method = create_test(gen)
    test_method.__name__ = 'test_decompiler_%d' % i
    setattr(TestDecompiler, test_method.__name__, test_method)



import unittest

from pony.orm import *
from pony import orm
from pony.orm.tests import setup_database, teardown_database

db = Database()


class User(db.Entity):
    name = Required(str)
    servers = Set("Server")


class Worker(User):
    pass


class Admin(Worker):
    pass

# And M:1 relationship with another entity
class Server(db.Entity):
    name = Required(str)
    user = Optional(User)


class Test(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        setup_database(db)
        with orm.db_session:
            Server(name='s1.example.com', user=User(name="Alex"))
            Server(name='s2.example.com', user=Worker(name="John"))
            Server(name='free.example.com', user=None)

    @classmethod
    def tearDownClass(cls):
        teardown_database(db)

    @db_session
    def test(self):
        qu = left_join((s.name, s.user.name) for s in db.Server)[:]
        for server, user in qu:
            if user is None:
                break
        else:
            self.fail()

